/*
A computer program called TMMLi
 (''Transmembrane protein analysis 
 using Maximum Likelihood'') has been developed to
implement an evolutionary model that 
permits analysis of phylogeny and transmembrane domain searching 
from aligned amino acid sequences.
 The model is related to that of
Dayhoff and coworkers but we distinguish eight categories of structural
environment: alpha-helix, beta-sheet, turn, loop and further
classifying each site according to its solvent accessibility i.e. buried or
exposed. The probabilistic model of sequence evolution for each of the eight
categories is described by hidden Markov chains. 
Both protein secondary structure prediction and phylogeny assessment can take advantage
of each other in term of significantly increased accuracy.
1) You need the following files:

tmml.c   C source code
tmml.h   header file
tmml.inp job file
eigen      eigenvectors
aaprob     aminoacid  stationary probabilities
model38    e.g model38 contains the hidden Markov transition matrix for the 38 state model

2) Aligned sequences should be placed in a file in sequential or multialigned format
3) Gaps and unknown amino acids are
represented by the "-" character. 
4) 10 characters are allowed for the sequence names
5) To compile this program type in a Unix environment type:
  cc tmml.c -lm -O -o tmml

*/
#include "eigen.c"
#include "tools.c"
#include "passml.h"
#include "drand48.c"

#define DEBUG         0
#define naa           20
#define LARGE         1.0e+18
#define LITTLE        1.0e-22
#define XACC          0.0001
#define X10ACC        0.001
#define X100ACC       0.01
#define X1000ACC      0.1
#define C_underflow   100
#define NG            4.154836887160073 
#define SIZE          3
#define NSIMUL       4 

int *thiscat,Ncat,Nstates, Nstru,get_cat,chosen_cat,*State2ss,sw1,if_underflow;
dssp dsspseq, accseq;
short Nspecie,Nnode,lengths,acc,i, j,k, l,col,post,anc,tree_rate,
      *order,*bestorder,*inputorder, protlen,stru,ntree,specie,help;
tree thistree,thattree;
FILE *infile, *outfile,*verbose;
boolean   lngths,faiboot, structure,accessibility,
       usertree,do_em, changed,nullo,sequential; 
char **aaseq,**aaseq1,ch,cec, *imat,*lmat,*umat,*hmm,*revn,inpf,outf,hmmacc[8],hmmcat[8],ord,boot,quit,MSW;
double ***Imat,***Umat,**Lmat,**Phmm, strfreq,tempval, Likeboot, Likeboot1;
double *AA,*UU,*VV,*Root,*ri,*space;
double ***Pq, ***Pr, **Ar,**Aq,***Bq,***Br,**aafreq,**pruninglike,**partiallike,*structfreq;
double BIGMLE[3];

/* ************************* */
main() {
recyclingbay();
loadingbay();
storagebay();
walkinforest();
 }
/* ******************************* *
    INPUT SECTION (PARAMETERS)
* ******************************* */
void recyclingbay() {
  int i,j,ii,iii,k,categ,stru1;
 char stru, utree,sei,lng,ch; 
  FILE *revf;
  FILE *hmmf;
  FILE *umatf;
  node *node1, *node2; 
/* starting conditions */
	do_em=false;
        stru='n';
	structure=false;
	faiboot=false;
        accessibility=false;
	usertree=false;
lngths=false;
	nullo=false;
	get_cat='a';
	ord='a';
	quit='n';
        boot=='a';	
	MSW='a';
	help='a';
/* setting procedure */
printf("\n   *********************************************************   \n");
printf("\n    TMml --> Transmembrane searching using ML      \n ");
printf("\n   *********************************************************   \n");
printf("this program infers branch lengths of an evolutionary tree \n");
printf("from a multialignment of amino acid sequences and predicts \n");
printf("transmembrane domains");
printf("\n\n\n\n - job file name: passml.inp  (ASCII format)\n");
printf("\n - verbose file name:   prolix  (all the trees evaluated)\n");
printf("\n\n\tPARAMETERS:\n");
        input_reset();
/*        input_insert("secondary structure y/n", &stru,'s'); */
/*        input_insert("accessibility y/n", &acc,'s'); */
        input_insert("number sequences ", &Nspecie,'i');
        input_insert("protein length ", &protlen,'i');
        input_insert("user tree y/n", &utree,'s');
        input_insert("use branch lengths y/n", &lng,'s'); 
        input_insert("sequence filename (please a short one) ", &inpf,'s');
        input_insert("sequential or interleaved s/i", &sei,'s');
        input_insert("output file (please a short one)", &outf,'s');
        input_insert("eigen file", &umat,'s');
        input_insert("model file",&hmm,'s');
        input_insert("aa stationary probabilities file",&revn,'s');
        input_insert("(a) HMM, (b) HMM &1st category is Coil exposed",&MSW,'s');
        input_insert("structure prediction: (a) No, (b) TM (transmembrane)_O(others),(c) TM_H_E_T(turn)_O",&get_cat,'s'); 
        input_insert("simulations: (a) No, (b) genera,(c) compare",&boot,'s'); 
	input_insert("quit y/n", &quit,'s'); 
        input_load("passml.inp");
        input();
        input_save("passml.inp");
/* ************************ */
if (quit=='y') exit(0);
	quit='n';
boot='a';
if (boot!='a') faiboot=true;
if (stru=='y') structure=true;
else structure=false;
if (acc=='y') accessibility=true;
else accessibility=false;
if (sei == 's') sequential=true;
else sequential=false;
	acc='n';
if (utree == 'y'){ usertree=true;
if (lng == 'y') {
tree_rate=1;
lngths=true;}
else lngths=false; }
else { 	usertree=false;
	lngths=false;
tree_rate=0;}
	infile=fopen(&inpf,"r");
	outfile=fopen(&outf,"w+");
	verbose=fopen("prolix","w+");
	hmmf=fopen(hmm,"r"); //-pac-
    fscanf(hmmf,"%d",&Nstates);
    fscanf(hmmf,"%d",&Nstru);
    Nnode = Nspecie * 2 - 2;
        order  = (short *)Malloc(Nnode*sizeof(short));
        bestorder  = (short *)Malloc(Nnode*sizeof(short));
        inputorder  = (short *)Malloc(Nnode*sizeof(short));
	fprintf(outfile,"Input parameters\n");
fprintf(outfile, "\n\n species        %4hd \n protein length %4hd \n structure         %c \n accessibility     %c \n tree              %c \n branch lengths    %c \n branches       %4hd \n", Nspecie,protlen,stru,acc,utree,lng,Nnode);
	Phmm=dmatrix(0,Nstates,0,Nstates);
	thiscat =ivector(0,Nstates);
	State2ss =ivector(0,Nstates);
	structfreq =dvector(0,Nstates);
        AA=dvector(0,naa*naa);
        UU=dvector(0,naa*naa);
        VV=dvector(0,naa*naa);
        Root=dvector(0,naa);
        ri=dvector(0,naa);
        space=dvector(0,2*naa);
	Lmat=dmatrix(0,Nstru,0,naa);
	aafreq=dmatrix(0,Nstru,0,naa);
	Imat=d3matrix(Nstru,naa,naa);
	Umat=d3matrix(Nstru,naa,naa);
	i=0;
boot='a';
for(ii=1;ii<=Nstru;ii++){
	fscanf(hmmf,"%s %s %d%d%lf",&hmmcat[ii],&hmmacc[ii],&categ,&stru1,&strfreq);
	structfreq[i]=strfreq;
for(j=i;j<=i+categ-1;j++){ 
	State2ss[j]=stru1;
structfreq[j]=0.0; }
structfreq[i]=strfreq;
	i+=categ; }

for(ii=0;ii<Nstates;ii++){
for(iii=0;iii<Nstates;iii++) {fscanf(hmmf,"%lf",&tempval);
	Phmm[ii][iii]=tempval;}}
if (get_cat!='a') {
while (chosen_cat != get_cat) { fscanf(hmmf,"%c",&chosen_cat);}
	fscanf(hmmf,"%d",&Ncat);
for(i=0;i<Ncat;i++) { fscanf(hmmf,"%d", &iii);
      fscanf(hmmf,"%d", &j);
for(ii=0;ii<j; ii++) { fscanf(hmmf,"%d", &k);
      thiscat[k] = iii;
printf("%d\t%d\n",k,thiscat[k]);
 }}}
fclose(hmmf);
umatf=fopen(umat,"r");   //-pac-
for(i=0;i<Nstru;i++){
for(ii=0;ii<=(naa-1);ii++){
for(iii=0;iii<=(naa-1);iii++) {fscanf(umatf,"%lf",&tempval);
AA[iii+naa*ii]=tempval;}}

if (eigen (1,AA,naa,Root,ri,UU,VV,space)) nrerror("error eigen\n");
xtoy(UU,VV,naa*naa);
matinv(VV,naa,naa,space);
for(ii=0;ii<naa;ii++){
for(iii=0;iii<naa;iii++){
Umat[i][ii][iii]=UU[iii+naa*ii];
Imat[i][ii][iii]=VV[iii+naa*ii];}

Lmat[i][ii]=Root[ii];}}
fclose(umatf);
revf=fopen(revn,"r");   //-pac-
for(j=0;j<Nstru;j++){ 
i=0;
while(i<naa) { ch='a'; 
if (ch>' ') {fscanf(revf,"%lf",&tempval);
	aafreq[j][i]=tempval;
	i++;}}
	aafreq[j][naa]=1.0; }
	fclose(revf);
if (structure) { dsspseq =cvector(0,protlen-1);
for(j=0;j<10;j++) ch = getc(infile);
	j=0;
while (j <= protlen-1) {
if ((ch = getc(infile))>='0') { dsspseq[j]=ch;
	j++; }}};
ch=' ';
if (accessibility) { accseq =cvector(0,protlen-1);
for(j=0;j<10;j++) ch = getc(infile);
	j=0;
while (j<=protlen-1) {fscanf(infile,"%c",&k);
if (k >= '2') ch='e';
else ch='b';
	accseq[j]=ch;
	j++; } }
aaseq=cdmatrix(0,Nspecie,0,protlen); 
if (boot=='b') aaseq1=cdmatrix(0,Nspecie,0,protlen);

thistree.pos = (node **)Malloc(Nnode*sizeof(node *));

for (i=0;i<Nspecie;i++){ 
thistree.pos[i]=(node *)Malloc(sizeof(node));
if (usertree) thistree.pos[i]->treename=false;
    thistree.pos[i]->leaf = true;
    thistree.pos[i]->aval = true;
    thistree.pos[i]->cond3 = 1;
if (tree_rate==0)    thistree.pos[i]->cond1 = true;
else thistree.pos[i]->cond1 = false;
    thistree.pos[i]->index1 = i+1;}

    thistree.start = thistree.pos[0];
for (i=Nspecie;i<Nnode;i++) {
node2 = NULL;

for (j=1;j<=3;j++) {
node1 = (node *)Malloc(sizeof(node));
node1->next = node2;
node2 = node1; 
node1->index1=i+1;
node1->leaf=false;
node1->cond3=false;
node1->aval=false;
if (tree_rate==0)    node1->cond1 = true;
else node1->cond1 = false;}

node1->next->next->next = node1;
thistree.pos[i] = node1;}


if (boot=='b') {
thattree.pos = (node **)Malloc(Nnode*sizeof(node *));
for (i=0;i<Nspecie;i++){
thattree.pos[i]=(node *)Malloc(sizeof(node));
if (usertree) thattree.pos[i]->treename=false;
    thattree.pos[i]->leaf = true;
    thattree.pos[i]->aval = true;
    thattree.pos[i]->cond3 = 1;
if (tree_rate==0)    thattree.pos[i]->cond1 = true;
else thattree.pos[i]->cond1 = false;
    thattree.pos[i]->index1 = i+1;}

    thattree.start = thattree.pos[0];
for (i=Nspecie;i<Nnode;i++) {
node2 = NULL;

for (j=1;j<=3;j++) {
node1 = (node *)Malloc(sizeof(node));
node1->next = node2;
node2 = node1;
node1->index1=i+1;
node1->leaf=false;
node1->cond3=false;
node1->aval=false;
if (tree_rate==0)    node1->cond1 = true;
else node1->cond1 = false;}

node1->next->next->next = node1;
thattree.pos[i] = node1;

}


free_dvector(AA,0,naa*naa);
free_dvector(UU,0,naa*naa);
free_dvector(VV,0,naa*naa);
free_dvector(Root,0,naa);
free_dvector(ri,0,naa);
free_dvector(space,0,2*naa);

   } /* end faiboot */

printf("\nparameters loaded");} 
/* ******************************* *
    INPUT SECTION (DATA)
* ******************************* */
void loadingbay()
{
/* char ch; */
short aaread, aatot,aanew,thisaa,thisseq,ok;
  aaread = 0;
  aatot = false;
while (aatot==0) {
aatot = 1;
if (eoln(infile)) {fscanf(infile, "%*[^\n]");
      getc(infile);}
      thisseq = 0;
while (thisseq < Nspecie) {
if (sequential||((!sequential) && aaread == 0)) {
for (thisaa = 0; thisaa < 10; thisaa++) 
  thistree.pos[thisseq]->seqname[thisaa] = getc(infile); }
if (sequential) thisaa = 0;
      else thisaa = aaread;
      ok = 1;
while ((ok==1) & !eof(infile)) {
if (sequential) ok = 1;
else ok=0;
while (((thisaa < (protlen)) & (!(eoln(infile) | eof(infile))))) {
	ch = getc(infile);
if (ch == '\n') ch = ' ';
if (ch == ' ' || (ch >= '0' && ch <= '9')) continue;
	uppercase(&ch);
/*	printf("%c \n",ch); */
test(ch,thisaa,thisseq);
	aaseq[thisseq][thisaa] = ch;
if ((ch == '?')||( ch == '-')||(ch == 'X')||(ch == 'J')) aaseq[thisseq][thisaa]='J'; 
thisaa++; }
if (!sequential) continue;
if (thisaa == (protlen)) ok=0;
else if (thisaa < protlen){
        fscanf(infile, "%*[^\n]");
        getc(infile); }}
if (!sequential && thisseq == 0) aanew = thisaa;
      fscanf(infile, "%*[^\n]");
      getc(infile);
     thisseq++;}
if (!sequential) { aaread = aanew;
      if (aaread == protlen) aatot=1;
      else aatot=0;
    } else {
    if(thisseq > Nspecie-1) aatot=1;
    else aatot=0;} } } 
/* ****************** */
void bootbay()
{
node *node1;
int i,j,k,l;
aatype b;
static int bootcount=1;
for (i = 0; i < Nspecie; i++) {
for (k = 0; k < protlen; k++) {
for (l = 0; l < Nstru; l++) {
for (b = A; (short)b <= (short)V; b = (aatype)((short)b + 1))
        thistree.pos[i]->sitevalue[k][l][(short)b - (short)A] = 0.0;
switch (aaseq[i][k]) {
         case 'A':
          thistree.pos[i]->sitevalue[k][l][(short)A - (short)A] = 1.0;
         break;
         case 'C':
          thistree.pos[i]->sitevalue[k][l][(short)C - (short)A] = 1.0;
         break;
         case 'D':
          thistree.pos[i]->sitevalue[k][l][(short)D - (short)A] = 1.0;
          break;
         case 'E':
          thistree.pos[i]->sitevalue[k][l][(short)E - (short)A] = 1.0;
          break;
         case 'G':
          thistree.pos[i]->sitevalue[k][l][(short)G - (short)A] = 1.0;
          break;
        case 'F':
          thistree.pos[i]->sitevalue[k][l][(short)F - (short)A] = 1.0;
          break;
        case 'H':
          thistree.pos[i]->sitevalue[k][l][(short)H - (short)A] = 1.0;
          break;
        case 'I':
          thistree.pos[i]->sitevalue[k][l][(short)I - (short)A] = 1.0;
          break;
        case 'K':
          thistree.pos[i]->sitevalue[k][l][(short)K - (short)A] = 1.0;
          break;
        case 'L':
          thistree.pos[i]->sitevalue[k][l][(short)L - (short)A] = 1.0;
          break;
        case 'M':
          thistree.pos[i]->sitevalue[k][l][(short)M - (short)A] = 1.0;
          break;
        case 'N':
          thistree.pos[i]->sitevalue[k][l][(short)N - (short)A] = 1.0;
          break;
        case 'P':
          thistree.pos[i]->sitevalue[k][l][(short)P - (short)A] = 1.0;
          break;
        case 'Q':
          thistree.pos[i]->sitevalue[k][l][(short)Q - (short)A] = 1.0;
          break;
        case 'R':
          thistree.pos[i]->sitevalue[k][l][(short)R - (short)A] = 1.0;
          break;
        case 'T':
          thistree.pos[i]->sitevalue[k][l][(short)T - (short)A] = 1.0;
          break;
        case 'S':
          thistree.pos[i]->sitevalue[k][l][(short)S - (short)A] = 1.0;
          break;
     case 'V':
          thistree.pos[i]->sitevalue[k][l][(short)V - (short)A] = 1.0;
          break;
        case 'W':
          thistree.pos[i]->sitevalue[k][l][(short)W - (short)A] = 1.0;
          break;
        case 'Y':
          thistree.pos[i]->sitevalue[k][l][(short)Y - (short)A] = 1.0;
          break;
        case 'J':  /* unknown aminoacid */
for (b = A; (short)b <= (short)V; b = (aatype)((short)b + 1))
          thistree.pos[i]->sitevalue[k][l][(short)b - (short)A] = 1.0;
         break;
        default:  ;
break; }}}}
if (usertree) { getc(infile);
    use_tree();}

if ((boot!='b')&&(boot!='a')) { 
fscanf(infile,"%lf",&Likeboot);
bootcount++;
}
}
/* ***************** */
void storagebay()
{
node *node1;
int i,j,k,l;
aatype b;
for (i = 0; i < Nspecie; i++){
    thistree.pos[i]->sitevalue=(aasite **)Malloc(protlen*sizeof(aasite *));
if (boot=='b') thattree.pos[i]->sitevalue=(aasite **)Malloc(protlen*sizeof(aasite *));
for (j=0;j<protlen;j++) {
    thistree.pos[i]->sitevalue[j]=(aasite *)Malloc(Nstru*sizeof(aasite));
if (boot=='b') thattree.pos[i]->sitevalue[j]=(aasite *)Malloc(Nstru*sizeof(aasite));

 }}
for (i=Nspecie;i<Nnode;i++){
	node1 = thistree.pos[i];
for (j=1;j <= 3; j++) {
    node1->sitevalue = (aasite **)Malloc(protlen*sizeof(aasite *));
for (k=0;k<protlen;k++) 
    node1->sitevalue[k] = (aasite *)Malloc(Nstru*sizeof(aasite));
    node1 = node1->next;}}
if (boot=='b') {
for (i=Nspecie;i<Nnode;i++){
        node1 = thattree.pos[i];
for (j=1;j <= 3; j++) {
    node1->sitevalue = (aasite **)Malloc(protlen*sizeof(aasite *));
for (k=0;k<protlen;k++)
    node1->sitevalue[k] = (aasite *)Malloc(Nstru*sizeof(aasite));
    node1 = node1->next;}}
}

for (i = 0; i < Nspecie; i++) {
for (k = 0; k < protlen; k++) { 
for (l = 0; l < Nstru; l++) {
for (b = A; (short)b <= (short)V; b = (aatype)((short)b + 1)) {
	thistree.pos[i]->sitevalue[k][l][(short)b - (short)A] = 0.0; 
if (boot=='b') thattree.pos[i]->sitevalue[k][l][(short)b - (short)A] = 0.0;
}
switch (aaseq[i][k]) {
         case 'A':
          thistree.pos[i]->sitevalue[k][l][(short)A - (short)A] = 1.0;
         break;
         case 'C':
          thistree.pos[i]->sitevalue[k][l][(short)C - (short)A] = 1.0;
         break;
         case 'D':
          thistree.pos[i]->sitevalue[k][l][(short)D - (short)A] = 1.0;
          break;
         case 'E':
          thistree.pos[i]->sitevalue[k][l][(short)E - (short)A] = 1.0;
          break;
         case 'G':
          thistree.pos[i]->sitevalue[k][l][(short)G - (short)A] = 1.0;
          break;
        case 'F':
          thistree.pos[i]->sitevalue[k][l][(short)F - (short)A] = 1.0;
          break;
        case 'H':
          thistree.pos[i]->sitevalue[k][l][(short)H - (short)A] = 1.0;
          break;
        case 'I':
          thistree.pos[i]->sitevalue[k][l][(short)I - (short)A] = 1.0;
          break;
        case 'K':
          thistree.pos[i]->sitevalue[k][l][(short)K - (short)A] = 1.0;
          break;
        case 'L':
          thistree.pos[i]->sitevalue[k][l][(short)L - (short)A] = 1.0;
          break;
        case 'M':
          thistree.pos[i]->sitevalue[k][l][(short)M - (short)A] = 1.0;
          break;
        case 'N':
          thistree.pos[i]->sitevalue[k][l][(short)N - (short)A] = 1.0;
          break;
        case 'P':
          thistree.pos[i]->sitevalue[k][l][(short)P - (short)A] = 1.0;
          break;
        case 'Q':
          thistree.pos[i]->sitevalue[k][l][(short)Q - (short)A] = 1.0;
          break;
        case 'R':
          thistree.pos[i]->sitevalue[k][l][(short)R - (short)A] = 1.0;
          break;
        case 'T':
          thistree.pos[i]->sitevalue[k][l][(short)T - (short)A] = 1.0;
          break;
        case 'S':
          thistree.pos[i]->sitevalue[k][l][(short)S - (short)A] = 1.0;
          break;
        case 'V':
          thistree.pos[i]->sitevalue[k][l][(short)V - (short)A] = 1.0;
          break;
        case 'W':
          thistree.pos[i]->sitevalue[k][l][(short)W - (short)A] = 1.0;
          break;
        case 'Y':
          thistree.pos[i]->sitevalue[k][l][(short)Y - (short)A] = 1.0;
          break;
        case 'J':  /* unknown aminoacid */
for (b = A; (short)b <= (short)V; b = (aatype)((short)b + 1)) 
          thistree.pos[i]->sitevalue[k][l][(short)b - (short)A] = 1.0;
         break;
        default:  ;        
break; }}}}
if (usertree) { getc(infile);
printf("\nuser_tree loaded\n");
    use_tree();}

if ((boot!='b')&(boot!='a')) {
fscanf(infile,"%lf",&Likeboot);
/* printf("\nurca %f\n",Likeboot); */
}

    printf("\ndata loaded\n");

}  
/* ************** */
void find1(c)
char *c;
{ do { 
if (eoln(infile)) { fscanf(infile, "%*[^\n]");
      getc(infile); }
    *c = getc(infile);
if (*c == '\n') *c = ' ';
	  } while (*c == ' '); }
/* ************** */
void find2(c,balancepar)
char c;
short *balancepar;
{  short mn;
	  mn = 0;
while (mn==0) {
if (c == ',')  mn=1;
else if (c == ')') { (*balancepar)--;
if ((*balancepar) == 0) {find1(&ch);}
		mn = 1; }
if (ch == ')') find1(&ch); }} 
/* ************************************************** */
void leaf_add(node1, nextnode,balancepar)
node *node1;
short *nextnode,*balancepar;
	{ node *node2;
	  short i, n,m;
	  char nameintree[10];
find1(&ch);
if (ch == '(') {(*balancepar)++;
	      (*nextnode)++;
	      node2 = thistree.pos[(*nextnode) - 1];
	      link(node1, node2);
	      leaf_add(node2->next,nextnode,balancepar);
	      find2(',',balancepar);
		leaf_add(node2->next->next,nextnode,balancepar);
find2(')',balancepar);
	  } else {
for (i = 0; i < 10; i++) nameintree[i] = ' ';
	    n = 1;
do {nameintree[n - 1] = ch;
 	printf("%c",nameintree[n-1]);
	ch = getc(infile);
	n++; } while (ch != ':' && ch != ',' && ch != ')' && n <= 10);
	printf("\n");   
	n = 1;
do {m = 0;
for (i = 0; i < 10; i++){
if (nameintree[i] == thistree.pos[n - 1]->seqname[i]&&(m==0)) m=0;
else m=1;}
if (m==0) {
if (thistree.pos[n - 1]->treename == false) thistree.pos[n - 1]->treename = true;} 
else n++;} 
while (!(n > Nspecie || m==0));
if (n > Nspecie) nrerror("number species in tree is greater than that in sequence file ");
	    link(thistree.pos[n- 1], node1);
if (thistree.start->index1 > n) thistree.start = thistree.pos[n - 1]; }
if ((tree_rate==1)&&(ch == ':')) { read_length(node1);}}
/* ******************************************* */
void read_length(node1)
	node *node1;
	{
	  double num, div;
      short decimal,dup;
	  dup = 1;
	  num= 0.0;
	  div = 1.0;
find1(&ch);
	  decimal = ch - '0';
while ( (unsigned short)decimal <=9 || ch == '.') {
if (ch != '.' ) { num *=  10.0;
        num += decimal;
if (dup==0) div *= 10.0;}
else dup = 0;
	    find1(&ch);
	    decimal = ch - '0'; }
            node1->rate = num / div;
	    node1->back->rate = node1->rate;
	    node1->cond1 = false;
	    node1->back->cond1 = false;} 
/* **************************** */
void use_tree()
	{
	  short nextnode, balancepar;
	  node *node1;
	  short i;
	  thistree.start = thistree.pos[Nspecie - 1];
	  find1(&ch);
if (ch != '(') return;
	  nextnode = Nspecie + 1;
	  node1 = thistree.pos[Nspecie];
	  balancepar = 1;
         leaf_add(node1, &nextnode,&balancepar) /* too much redundant */;
        node1 = node1->next;
        find2(',',&balancepar); 
        leaf_add(node1, &nextnode,&balancepar);
        node1 = node1->next;
        find2(',',&balancepar); 
	leaf_add(node1, &nextnode,&balancepar);
	find2(')',&balancepar);
	fscanf(infile, "%*[^\n]");
	getc(infile);}
/* ******************************* *
    PROCESS SECTION 
 ********************************* */
void copynode(c, d)
node *c, *d;
{
  int i, j;

  memcpy(d->seqname, c->seqname, sizeof(10));
  for (i=0;i<protlen;i++) {
    for (j = 0; j < Nstru; j++) {
      memcpy(d->sitevalue[i][j],c->sitevalue[i][j], sizeof(aasite)); }}
  d->leaf = c->leaf;
  d->index1 = c->index1; 
  d->aval = c->aval;
  d->treename = c->treename;
  d->rate = c->rate;
  d->cond1 = c->cond1;
  d->cond2 = c->cond2;
  d->cond3 = c->cond3;
  d->cond4 = c->cond4;
}  /* copynode */

/* ****************************** */
void copytree(a, b)
tree *a, *b;
{
  short i, j=0;
  node *p, *q;

  for (i = 0; i < Nspecie; i++) {
    copynode(a->pos[i], b->pos[i]);
    if (a->pos[i]->back) {
      if (a->pos[i]->back == a->pos[a->pos[i]->back->index1 - 1])
        b->pos[i]->back = b->pos[a->pos[i]->back->index1 - 1];
      else if (a->pos[i]->back == a->pos[a->pos[i]->back->index1 - 1]->next)
        b->pos[i]->back = b->pos[a->pos[i]->back->index1 - 1]->next;
      else
        b->pos[i]->back = b->pos[a->pos[i]->back->index1 - 1]->next->next;
    }
    else b->pos[i]->back = NULL;
  }
  for (i = Nspecie; i < Nnode; i++) {
    p = a->pos[i];
    q = b->pos[i];
    for (j = 1; j <= 3; j++) {
      copynode(p, q);
      if (p->back) {
        if (p->back == a->pos[p->back->index1 - 1])
          q->back = b->pos[p->back->index1 - 1];
        else if (p->back == a->pos[p->back->index1 - 1]->next)
          q->back = b->pos[p->back->index1 - 1]->next;
        else
          q->back = b->pos[p->back->index1 - 1]->next->next;
      }
      else
        q->back = NULL;
      p = p->next;
      q = q->next;
    }
  }
  b->start = a->start;
}  /* copy */

/* ***********************************  */

void leaf2root(node1,tre)
tree *tre;
node *node1;
{
short i,ii,k, j;
aatype b,bb;
double rate1, rate2; 
double sum1,sum2,sumc;
node *node2, *node3;
aasite leaf1,leaf2, intnode;
  sumc=0.0;
  node2 = node1->next->back; 
  node3 = node1->next->next->back;
  rate1 = node2->rate;
  rate2 = node3->rate;
Bq=d3matrix(Nstru,naa,naa);
Br=d3matrix(Nstru,naa,naa);
Ar=dmatrix(0,Nstru,0,naa);
Aq=dmatrix(0,Nstru,0,naa);
for (j = 0; j < Nstru; j++) {
for (i = 0; i < naa; i++) {
            Aq[j][i]=(double)exp(rate1*Lmat[j][i]);
            Ar[j][i]=(double)exp(rate2*Lmat[j][i]); }}
for (j = 0; j <Nstru; j++) {
for (i = 0; i < naa; i++) {
for (ii = 0; ii < naa; ii++) {
	Bq[j][i][ii]=Umat[j][i][ii]*Aq[j][ii];
	Br[j][i][ii]=Umat[j][i][ii]*Ar[j][ii]; }}}
free_dmatrix(Aq,0,Nstru,0,naa);
free_dmatrix(Ar,0,Nstru,0,naa);
Pq=d3matrix(Nstru,naa+1,naa+1);
Pr=d3matrix(Nstru,naa+1,naa+1);

for (j = 0; j < Nstru; j++) {
for (i = 0; i < naa; i++) {
	Pq[j][i][i]=0.0;
	Pr[j][i][i]=0.0;
for (ii = 0; ii < naa; ii++) {
	Pq[j][i][i]+=Bq[j][i][ii]*Imat[j][ii][i];
	Pr[j][i][i]+=Br[j][i][ii]*Imat[j][ii][i]; }
for (k = i+1; k < naa; k++) {
	Pr[j][i][k]=0.0;
	Pq[j][i][k]=0.0;
for (ii = 0; ii < naa; ii++) {
	Pq[j][i][k]+=Bq[j][i][ii]*Imat[j][ii][k];
	Pr[j][i][k]+=Br[j][i][ii]*Imat[j][ii][k]; }
if (Pq[j][i][k]<LITTLE) Pq[j][i][k]=LITTLE;
if (Pr[j][i][k]<LITTLE) Pr[j][i][k]=LITTLE;
	Pq[j][k][i]=Pq[j][i][k]*aafreq[j][i]/aafreq[j][k];
	Pr[j][k][i]=Pr[j][i][k]*aafreq[j][i]/aafreq[j][k]; }
	Pq[j][i][naa]=1.0;
	Pr[j][i][naa]=1.0;
	Pq[j][naa][i]=aafreq[j][i];
	Pr[j][naa][i]=aafreq[j][i];}
	Pq[j][naa][naa]=1.0;
	Pr[j][naa][naa]=1.0;  }
for (i = 0; i < protlen; i++) {
for (j = 0; j <Nstru; j++) {
	memcpy(leaf1, node2->sitevalue[i][j], sizeof(aasite));
	memcpy(leaf2, node3->sitevalue[i][j], sizeof(aasite));
	sumc=0.0;
for (bb = A; (short)bb <= (short)V; bb = (aatype)((short)bb + 1)){
	sum1=0.0;
	sum2=0.0;
for (b = A; (short)b <= (short)V; b = (aatype)((short)b + 1)){
	sum1 += Pq[j][(short)bb - (short)A][(short)b - (short)A] *  leaf1[(short)b - (short)A];
	sum2 += Pr[j][(short)bb - (short)A][(short)b - (short)A] *  leaf2[(short)b - (short)A]; }
	sumc+=sum1*sum2;
	intnode[(short)bb - (short)A] =sum1*sum2; }
	memcpy(node1->sitevalue[i][j], intnode, sizeof(aasite)); } }
	free_d3matrix(Pq,Nstru,naa+1,naa+1);
	free_d3matrix(Pr,Nstru,naa+1,naa+1);
	free_d3matrix(Bq,Nstru,naa,naa);
	free_d3matrix(Br,Nstru,naa,naa);
 }
/* ********************** */
void preprocessing(tre)
tree *tre;
{
short i,j;
node *node1, *node2,*node3;
aatype b;
for (i=Nspecie;i<Nnode;i++) {
node1=thistree.pos[i];
node2 = node1->next->back;
node3 = node1->next->next->back;
for (i = 0; i < protlen; i++) {
for (j = 0; j < Nstru; j++) {
for (b = A; (short)b <= (short)V; b = (aatype)((short)b + 1))
node1->sitevalue[i][j][(short)b - (short)A] =0.5 * (node2->sitevalue[i][j][(short)b - (short)A] + node3->sitevalue[i][j][(short)b - (short)A]);}}}
if (node1->cond1) {
if (node1->cond2) node1->rate=100.0; 
else node1->rate=0.0;
}}
/* **************************** */
void call_leaf2root(tre)
tree *tre;
{short i,j;
node *node1;

for (i=Nspecie;i<Nnode;i++) {
    node1 =thistree.pos[i];
for (j=1;j<=3;j++) {
if ((node1->next->back->cond3)&&(node1->next->next->back->cond3)&&(!node1->cond3)){
leaf2root(node1,tre);
node1->cond3=true; }
node1 = node1->next;}}}
/* ******************* */
double calc_like(node1,tre)
	node *node1;
tree *tre;
{ double tot_if_underflow;
double sum,rate2,sum1,sum2;
short   i,ii, j,pj, Ind_cat;
node *node2,*node3;
aasite aasite1,aasite2;
aatype b,bb;
double xtwomacro, xmacro,domicro;
	pruninglike=dmatrix(0,protlen,0,Nstru);
	xtwomacro= exp(2.0*C_underflow);
	xmacro = exp(1.0*C_underflow);
	domicro= exp(-1.0*C_underflow);
    tot_if_underflow=1.0*C_underflow;
	sum = 0.0;
	node1->rate = -1.0*C_underflow*log(node1->rate);
	node1->back->rate = node1->rate; 
	node2 = node1->back;
for (i=Nspecie;i<Nnode;i++) {
node2=thistree.pos[i];
for (j=1;j<=3;j++) {
node2->cond3=false;
node2=node2->next;}}

for (j = 0; j < Nspecie*5; j++) {
call_leaf2root(&tre);} 
rate2 =(node1->rate);

	Bq=d3matrix(Nstru,naa,naa);
	Aq=dmatrix(0,Nstru,0,naa); 
for (j = 0; j < Nstru; j++) {
for (i = 0; i < naa; i++) {
	Aq[j][i]=exp(rate2*Lmat[j][i]); }}
for (j = 0; j < Nstru; j++) {
for (i = 0; i < naa; i++) {
for (ii = 0; ii < naa; ii++) {
	Bq[j][i][ii]=Umat[j][i][ii]*Aq[j][ii]; }}}

free_dmatrix(Aq,0,Nstru,0,naa);
Pq=d3matrix(Nstru,naa+1,naa+1);

for (j = 0; j < Nstru; j++) {
for (i = 0; i < naa; i++) {
	Pq[j][i][i]=0.0;
for (ii=0;ii<naa;ii++) Pq[j][i][i]+=Bq[j][i][ii]*Imat[j][ii][i];
for (k = i+1; k < naa; k++) {
	Pq[j][i][k]=0.0;
for (ii = 0; ii < naa; ii++) {
	Pq[j][i][k]+=Bq[j][i][ii]*Imat[j][ii][k]; }
if (Pq[j][i][k]<LITTLE) Pq[j][i][k]=LITTLE;
	Pq[j][k][i]=Pq[j][i][k]*aafreq[j][i]/aafreq[j][k]; }
	Pq[j][i][naa]=1.0;
	Pq[j][naa][i]=aafreq[j][i]; }
	Pq[j][naa][naa]=1.0; }

for (i = 0; i < protlen; i++) {
for (j = 0; j < Nstru; j++) {
    pruninglike[i][j]=0.0;
    memcpy(aasite1, node1->sitevalue[i][j], sizeof(aasite));
    memcpy(aasite2, node1->back->sitevalue[i][j], sizeof(aasite));
for (bb = A; (short)bb <= (short)V; bb = (aatype)((short)bb + 1)){
	sum1=0.0;
for (b = A; (short)b <= (short)V; b = (aatype)((short)b + 1)){
	sum1 += Pq[j][(short)bb - (short)A][(short)b - (short)A]* aasite1[(short)b - (short)A];}
if (sum1<=0.0) sum1=LITTLE;
	pruninglike[i][j] +=sum1*aasite2[(short)bb - (short)A]*aafreq[j][(short)bb - (short)A];}}} 
if_underflow=0;
	free_d3matrix(Pq,Nstru,naa+1,naa+1);
	free_d3matrix(Bq,Nstru,naa,naa);
switch (MSW) {

	case 'b': 
/* HMM | coil exposed  */
for (j=0;j<Nstates;j++) {
if (j==18) partiallike[0][j]=pruninglike[0][State2ss[j]];
else partiallike[0][j]=0; }
 
tot_if_underflow=0.0;

for (i=1;i< protlen; i++) {
if_underflow=0;
for (j=0;j<Nstates;j++){partiallike[i][j]=0.0;

for (k=0;k<Nstates;k++) {partiallike[i][j]+=partiallike[i-1][k]*Phmm[k][j];}

partiallike[i][j]*=pruninglike[i][State2ss[j]];
        if (partiallike[i][j]<domicro) if_underflow+=1;}

if(if_underflow==Nstates) {
for(j=0;j<Nstates;j++) partiallike[i][j] *= xtwomacro;
tot_if_underflow+=(2.0*C_underflow);}
if_underflow=0;}
 sum=0.0;
for (j=0;j<Nstates;j++) sum+=(partiallike[protlen-1][j]);
        break;

	case 'a':
/* HMM */
for (j=0;j<Nstates;j++) {
partiallike[0][j]=pruninglike[0][State2ss[j]]*structfreq[j]*xmacro;
if (partiallike[0][j]<domicro) if_underflow+=1;}
tot_if_underflow=1.0*C_underflow;
if (if_underflow==Nstates) {
for(j=0;j<Nstates;j++) partiallike[0][j] *= xtwomacro;
tot_if_underflow+=(2.0*C_underflow);
if_underflow=0;}
for (i=1;i< protlen; i++) {
if_underflow=0; 
for (j=0;j<Nstates;j++){partiallike[i][j]=0.0;
for (k=0;k<Nstates;k++) {partiallike[i][j]+=partiallike[i-1][k]*Phmm[k][j];}
partiallike[i][j]*=pruninglike[i][State2ss[j]];
if (partiallike[i][j]<domicro) if_underflow+=1;}
if(if_underflow==Nstates) {
for(j=0;j<Nstates;j++) partiallike[i][j] *= xtwomacro;
tot_if_underflow+=(2.0*C_underflow);}
if_underflow=0;}
 sum=0.0;
for (j=0;j<Nstates;j++) sum+=(partiallike[protlen-1][j]);
break;


case 'e':
/* structure */
j=0;
for (i=0;i<protlen;i++){
switch (dsspseq[i]) {
          case 'H':
          Ind_cat=0;         
          break;
          case 'E':
          Ind_cat=10;     
          break;
          case 'T':
          Ind_cat=16;
          break;
          case 'S':
          Ind_cat=16;
          break;
          case 'C' :
          Ind_cat=18;
          break;
          case 'B' :
          Ind_cat=18;
          break;
          case 'G':
          Ind_cat=18;
          break;
          case 'I':
          Ind_cat=18;
          break;
          case '.':
          Ind_cat=18;
          break; }
if( accseq[i]=='e') Ind_cat+=(Nstates/2);
if (i==0) partiallike[0][Ind_cat]=pruninglike[0][State2ss[Ind_cat]]*structfreq[State2ss[Ind_cat]];
else {partiallike[i][Ind_cat]=partiallike[i-1][pj]*pruninglike[i][State2ss[Ind_cat]];}
pj=Ind_cat;}
break;}

	sum=log(sum)-tot_if_underflow;  
	node1->rate= exp(-1.0*X100ACC*(node1->rate));
	node1->back->rate=node1->rate; 
	free_dmatrix(pruninglike,0,protlen,0,Nstru); 
    return sum;} 
/* ******************* */
void calc_em(node1,BIGMLE,tre)
node *node1;
double BIGMLE[];
tree *tre;
{int sw3,k; 
double inc, incstep, old_rate,smallmle[4];
static int sw1=0;
node1->rate= exp(-1.0*X100ACC* (node1->rate));
node1->back->rate=node1->rate;
         smallmle[0] = BIGMLE[2];
         old_rate = node1->rate;
if (node1->cond1) inc = X100ACC*min(node1->rate, 1.0 - node1->rate);
else inc=0;
while((old_rate + inc) > 1.0 || (old_rate - inc < 0.0)) inc *= 0.5;
if(sw1 ==0)  {
          smallmle[2] = calc_like(node1,tre);
          smallmle[0] = smallmle[2];
          sw1=1; }
else smallmle[2] = smallmle[0];
          node1->rate += inc * old_rate;
          node1->back->rate =node1->rate;
          smallmle[3] = calc_like(node1,tre);
          node1->rate -= 2*inc*old_rate;
          node1->back->rate =node1->rate;
          smallmle[1] =  calc_like(node1,tre);
if(smallmle[3]+smallmle[1] - 2*smallmle[2] < 0.0)
          incstep =  (-0.5)* inc*(smallmle[3]-smallmle[1])/(smallmle[3] +smallmle[1] - 2*smallmle[2]);
else { if(node1->cond1) incstep = X1000ACC* (smallmle[3] - smallmle[2])/inc;
else incstep=0; }
k=0;
sw3 =1;  
while(sw3>0) {
if((old_rate + incstep > 1.0)||(old_rate +incstep <0.0)) incstep *= 0.5;
else {    node1->rate = old_rate + incstep;
          node1->back->rate = node1->rate;
          BIGMLE[2] = calc_like(node1,tre);
if(BIGMLE[2] >= smallmle[0] ||  node1->rate == old_rate) sw3 = 0;
else incstep *= 0.5; }
}
node1->rate = -1.0*C_underflow*log(node1->rate);
node1->back->rate = node1->rate;}
/* ****************   */
void call_calc_em(like2,tre)
double *like2;
tree *tre;
{
short i,j,k;
node *node1;
k=0;
for (i=Nspecie;i<Nnode;i++) {
node1 =thistree.pos[i];
for (j=1;j<=3;j++) {
node1->cond4=0;
node1=node1->next;}}

for (i=Nspecie;i<Nnode;i++) {
node1 =thistree.pos[i];
for (j=1;j<=3;j++) {
if ((node1->cond2)&&(node1->cond4==0)) {
calc_em(node1,like2,&tre);
/* printf(" %15.5f\t %d\t%d\n", node1->rate*NG,node1->index1,node1->back->index1); */
node1->cond4=1;
node1=node1->next;}}}
}

/* ******************** */
void call_calc_like(like2,tre)
double like2;
tree *tre;
{
short i,j,k;
node *node1;
k=0;
for (i=Nspecie;i<Nnode;i++) {
node1 =thistree.pos[i];
for (j=1;j<=3;j++) {
like2=calc_like(node1,tre);
node1=node1->next;}}
}
/* ******************** */
void output(node1,endprint)
node *node1;
boolean endprint;
	{short i;
	node *node2;
  node2 = node1->back;
if (endprint) fprintf(outfile,"%4hd      ", node2->index1);
if (node1->leaf) {
if (endprint) {for (i=0;i<10;i++) putc(node1->seqname[i],outfile);}
	  } else {
if (endprint)fprintf(outfile,"%4hd      ", node1->index1);  }
/* if (endprint) {fprintf(outfile, "%15.5f", node1->rate); */
if (endprint) {fprintf(outfile, "%15.5f", node2->rate*NG); 
	  putc('\n', outfile);}
if (!node1->leaf) {output(node1->next->back,endprint);
	    output(node1->next->next->back,endprint); } }
/* *************************** */ 
void prediction(tre)
tree *tre;
{
int i,j,jj,k,l, *Tcat;
aatype b;
double Pa_b,*PP3, *PP2,**PP1,**Pout,Eaa,tempval1,*Pstru,tot,Ess;
/*PP1,PP2,PP3 numerator and denominator of the formula 30-31
in JMB paper by Goldman et al.*/
        Pout=dmatrix(0,Ncat,0,protlen);        
        PP1=dmatrix(0,Nstates,0,protlen);
	PP2=dvector(0,Nstates);
	PP3=dvector(0,Nstates);
	Pstru=dvector(0,protlen);
	Tcat=ivector(0,protlen);

/* the entropy gives an idea of how much the signal is concentrated;
the ratio alpha-helix/beta-sheet estimates the helix or beta sheet 
propensity */
for(i=(protlen-1) ; i>=0;i--) { tot = 0.0;
for(k=0;k< Ncat;k++) {
Pout[k][i]=0.0;
}
for(j=0; j< Nstates; j++) tot += partiallike[i][j];
for(j=0; j< Nstates; j++){
if (tot>0.0)  PP3[j] = partiallike[i][j]/tot;   
else PP3[j] = partiallike[i][j];}
if(i!=(protlen-1)){
for(jj= 0; jj< Nstates; jj++){ PP2[jj] = 0.0;
for(j=0; j< Nstates; j++) PP2[jj] += PP3[j]*Phmm[j][jj]; }}
for(j= 0; j< Nstates;j++){
if(i != (protlen-1)){  PP1[j][i] = 0.0;
for(jj = 0; jj < Nstates; jj++){
if(PP2[jj] > LITTLE){
	PP1[j][i] += (PP1[jj][i+1] *Phmm[j][jj])/ PP2[jj];}}
    PP1[j][i] *= PP3[j];
         } else PP1[j][i] = PP3[j];

Pout[thiscat[j]][i]+=PP1[j][i];}

for(k=0; k<Ncat;k++) {
if(k == 0) { Tcat[i] = 0;
Pstru[i]=Pout[0][i];
         } else {
if (Pout[k][i] > Pstru[i]) { 
Pstru[i] =Pout[k][i];
	Tcat[i] = k; } }}} 
fprintf(outfile,"\n\nSecondary Structure prediction \n\n ");
fprintf(outfile,"\nNumber of predicted categories %d\n\n", Ncat);
for (i=0;i<protlen;i++){
fprintf(outfile,"\n%5d ",i+1);

Pa_b=0.0;
Ess=0.0;
for (k=0;k<Ncat;k++){

if (Pout[k][i]>0.0) Ess-=Pout[k][i]*log(Pout[k][i]);
else Ess=0.0;
fprintf(outfile,"\t%8.6lf ",Pout[k][i]);}
if (Pout[1][i]>0.0) Pa_b=Pout[0][i]/Pout[1][i];
else Pa_b=-9999.9999;
fprintf(outfile,"\t\t");
if (structure) fprintf(outfile,"%c ", dsspseq[i]);
/* fprintf(outfile,"%8.6lf ", Pa_b);
fprintf(outfile,"\t%8.6lf ", Ess);*/ } 
	free_dmatrix(PP1,0,Nstates,0,protlen);
	free_dmatrix(Pout,0,Ncat,0,protlen);
	free_dvector(PP2,0,Nstates);
	free_dvector(PP3,0,Nstates);
	free_dvector(Pstru,0,protlen);
	free_ivector(Tcat,0,protlen);
}
/* ************************** */
changenode(node1,c)
node *node1;
int c[];
{
int aa, i,l,k,ii,in,iii,kk;
aatype b,bb;
double rnumb,sumc,rate1,sum1;
double  aapvet[21];
node *p, *node2;
aasite leaf1,intnode;
char aas;
srand ((unsigned)time(NULL));
node2 = node1->next->back;

Pq=d3matrix(Nstru,naa+1,naa+1);
Bq=d3matrix(Nstru,naa,naa);
Aq=dmatrix(0,Nstru,0,naa);
kk=c[2];
l=c[1];
sumc=0.0;

rnumb=drand48();  
p=node1->next->back;;  
in=p->index1-1; 
rate1=p->rate;
for (j = 0; j < Nstru; j++) {
for (i = 0; i < naa; i++) {
           Aq[j][i]=(double)exp(rate1*Lmat[j][i]);}}
for (j = 0; j <Nstru; j++) {
for (i = 0; i < naa; i++) {
for (ii = 0; ii < naa; ii++) {
 Bq[j][i][ii]=Umat[j][i][ii]*Aq[j][ii]; }}}
for (j = 0; j < Nstru; j++) {
for (i = 0; i < naa; i++) {
 Pq[j][i][i]=0.0;
for (ii = 0; ii < naa; ii++) {
 Pq[j][i][i]+=Bq[j][i][ii]*Imat[j][ii][i];}
for (k = i+1; k < naa; k++) {
Pq[j][i][k]=0.0;
for (ii = 0; ii < naa; ii++) {
 Pq[j][i][k]+=Bq[j][i][ii]*Imat[j][ii][k];}

if (Pq[j][i][k]<LITTLE) Pq[j][i][k]=LITTLE;
 Pq[j][k][i]=Pq[j][i][k]*aafreq[j][i]/aafreq[j][k];}
 Pq[j][i][naa]=1.0;
 Pq[j][naa][i]=aafreq[j][i];}
 Pq[j][naa][naa]=1.0;}

memcpy(leaf1, node1->sitevalue[kk][l], sizeof(aasite));
  sumc=0.0;
for (bb = A; (short)bb <= (short)V; bb = (aatype)((short)bb + 1)){
 sum1=0.0;
for (b = A; (short)b <= (short)V; b = (aatype)((short)b + 1)){
sum1 += Pq[l][(short)bb - (short)A][(short)b - (short)A]*leaf1[(short)b - (short)A];  
 }
 sumc+=sum1;
 intnode[(short)bb - (short)A] =sum1; }


sumc=0.0;
for (bb = A; (short)bb <= (short)V; bb = (aatype)((short)bb + 1)) {
sumc+=intnode[(short)bb - (short)A];
}

for (bb = A; (short)bb <= (short)V; bb = (aatype)((short)bb + 1)) {
intnode[(short)bb - (short)A] =intnode[(short)bb - (short)A]/sumc;  
}  


i=in;


for (aa = 0; aa <=20 ; aa++) aapvet[aa]=0.0;
aa=0;
aapvet[aa]=intnode[(short)A - (short)A];
for (b = R; (short)b <= (short)V; b = (aatype)((short)b + 1)){
aa++;
aapvet[aa] =aapvet[aa-1] +(intnode[(short)b - (short)A]);
}

aapvet[20]=1.0;
/*
for (aa = 0; aa <20 ; aa++) {
printf("%f\n",aapvet[aa]); 
}

*/
k=kk; 

if (i<Nspecie){
if ((rnumb >0.0) && (rnumb<aapvet[0])) aaseq1[i][k]='A';
else if (rnumb <aapvet[1]) aaseq1[i][k]='R';
else if (rnumb <aapvet[2]) aaseq1[i][k]='N';
else if (rnumb <aapvet[3]) aaseq1[i][k]='D';
else if (rnumb <aapvet[4]) aaseq1[i][k]='C';
else if (rnumb <aapvet[5]) aaseq1[i][k]='Q';
else if (rnumb <aapvet[6]) aaseq1[i][k]='E';
else if (rnumb <aapvet[7]) aaseq1[i][k]='G';
else if (rnumb <aapvet[8]) aaseq1[i][k]='H';
else if (rnumb <aapvet[9]) aaseq1[i][k]='I';
else if (rnumb <aapvet[10]) aaseq1[i][k]='L';
else if (rnumb <aapvet[11]) aaseq1[i][k]='K';
else if (rnumb <aapvet[12]) aaseq1[i][k]='M';
else if (rnumb <aapvet[13]) aaseq1[i][k]='F';
else if (rnumb <aapvet[14]) aaseq1[i][k]='P';
else if (rnumb <aapvet[15]) aaseq1[i][k]='S';
else if (rnumb <aapvet[16]) aaseq1[i][k]='T';
else if (rnumb <aapvet[17]) aaseq1[i][k]='W';
else if (rnumb <aapvet[18]) aaseq1[i][k]='Y';
else if (rnumb <aapvet[19]) aaseq1[i][k]='V';
else aaseq1[i][k]='J';
}

if ((rnumb >0.0) && (rnumb<aapvet[0])) aas='A';
else if (rnumb <aapvet[1]) aas='R';
else if (rnumb <aapvet[2]) aas='N';
else if (rnumb <aapvet[3]) aas='D';
else if (rnumb <aapvet[4]) aas='C';
else if (rnumb <aapvet[5]) aas='Q';
else if (rnumb <aapvet[6]) aas='E';
else if (rnumb <aapvet[7]) aas='G';
else if (rnumb <aapvet[8]) aas='H';
else if (rnumb <aapvet[9]) aas='I';
else if (rnumb <aapvet[10]) aas='L';
else if (rnumb <aapvet[11]) aas='K';
else if (rnumb <aapvet[12]) aas='M';
else if (rnumb <aapvet[13]) aas='F';
else if (rnumb <aapvet[14]) aas='P';
else if (rnumb <aapvet[15]) aas='S';
else if (rnumb <aapvet[16]) aas='T';
else if (rnumb <aapvet[17]) aas='W';
else if (rnumb <aapvet[18]) aas='Y';
else if (rnumb <aapvet[19]) aas='V';
else aas='J';

for (l=0;l<Nstru; l++) {
for (b = A; (short)b <= (short)V; b = (aatype)((short)b + 1))
        p->sitevalue[k][l][(short)b - (short)A] = 0.0;
       switch (aas) {
         case 'A':
          p->sitevalue[k][l][(short)A - (short)A] = 1.0;
         break;
         case 'C':
          p->sitevalue[k][l][(short)C - (short)A] = 1.0;
         break;
         case 'D':
          p->sitevalue[k][l][(short)D - (short)A] = 1.0;
          break;
         case 'E':
          p->sitevalue[k][l][(short)E - (short)A] = 1.0;
          break;
         case 'G':
          p->sitevalue[k][l][(short)G - (short)A] = 1.0;
          break;
        case 'F':
          p->sitevalue[k][l][(short)F - (short)A] = 1.0;
          break;
        case 'H':
          p->sitevalue[k][l][(short)H - (short)A] = 1.0;
          break;
        case 'I':
          p->sitevalue[k][l][(short)I - (short)A] = 1.0;
          break;
        case 'K':
          p->sitevalue[k][l][(short)K - (short)A] = 1.0;
          break;
        case 'L':
          p->sitevalue[k][l][(short)L - (short)A] = 1.0;
          break;
        case 'M':
          p->sitevalue[k][l][(short)M - (short)A] = 1.0;
          break;
        case 'N':
          p->sitevalue[k][l][(short)N - (short)A] = 1.0;
          break;
        case 'P':
          p->sitevalue[k][l][(short)P - (short)A] = 1.0;
          break;
        case 'Q':
          p->sitevalue[k][l][(short)Q - (short)A] = 1.0;
          break;
        case 'R':
          p->sitevalue[k][l][(short)R - (short)A] = 1.0;
          break;
        case 'T':
          p->sitevalue[k][l][(short)T - (short)A] = 1.0;
          break;
        case 'S':
          p->sitevalue[k][l][(short)S - (short)A] = 1.0;
          break;
        case 'V':
          p->sitevalue[k][l][(short)V - (short)A] = 1.0;
          break;
        case 'W':
          p->sitevalue[k][l][(short)W - (short)A] = 1.0;
          break;
        case 'Y':
          p->sitevalue[k][l][(short)Y - (short)A] = 1.0;
          break;
        case 'J':  
 for (b = A; (short)b <= (short)V; b = (aatype)((short)b + 1))
          p->sitevalue[k][l][(short)b - (short)A] = 1.0;
         break;
        default:  ;
        break;
}}

 free_d3matrix(Pq,Nstru,naa+1,naa+1);
 free_d3matrix(Bq,Nstru,naa,naa);
 free_dmatrix(Aq,0,Nstru,0,naa);

}

/* ****************************** */
void set_true(tre)
tree *tre;
{short i, j;
  node *node1;
for (i = 0; i < Nspecie; i++) {
    thistree.pos[i]->leaf==true;
    thistree.pos[i]->cond2=true; }
for (i=Nspecie;i<Nnode;i++) {
    node1 = thistree.pos[i];
for (j=1;j<=3;j++) {node1->cond2=true;
        node1->leaf=false;
      node1=node1->next;}}}
/* ****************************** */
void set_true2(tre)
tree *tre;
{short i, j;
  node *node1;
for (i= 0; i < Nspecie; i++) {
    thistree.pos[i]->leaf==true;
 if (thistree.pos[i]->cond1) thistree.pos[i]->rate=100.0;
    thistree.pos[i]->cond2=true; }
for (i=Nspecie;i<Nnode;i++) {
    node1 = thistree.pos[i];
for (j=1;j<=3;j++) {node1->cond2=true;
if (node1->cond1) node1->rate=100.0;
        node1->leaf=false;
      node1=node1->next;}}}
/* ****************************** */
void output3(node1,endprint)
node *node1;
boolean endprint;
{short i, n;
char c;
double rate1;
if (node1->leaf) {   n = 0;
for (i = 0; i < 10; i++) { c = node1->seqname[i];
if (c!=' ') {
if (endprint) putc(c, verbose);} }
          } else {
if (endprint) putc('(', verbose);
            output3(node1->next->back,endprint);

if (endprint) putc(',', verbose);
            output3(node1->next->next->back,endprint);
if (node1 == thistree.start->back) {

if (endprint) putc(',', verbose);
            output3(node1->back,endprint); }

if (endprint) putc(')', verbose);  }
            rate1 = node1->rate;
if (node1 != thistree.start->back) {
/* if (endprint) fprintf(outfile, ":%lf", rate1*NG); */
/* if (endprint) fprintf(verbose, ":%lf", rate1); */
}
}

/* ************************** */
void bootstrap(tre)
tree *tre;
{
short i,j,k;
node *node1;
int aa, a[3];
double xpv;
boolean pippo;
static count1=1;
k=0;
/* banana  */

/* c 
/* si legge un file multiplo 
nome specie, sequenza, albero
e likelihood. 
si calcola e si stampa  
la differenza tra la likelihood 
letta e quella calcolata col nuovo modello */

if (boot!='b') { 
loadingbay();
bootbay(); 
thistree.start=thistree.pos[Nspecie-1];
thistree.start->rate= exp(-1.0*X100ACC* (thistree.start->rate)); 
Likeboot1=calc_like(thistree.start,&thistree);
fprintf(verbose," %lf\t%lf\t%lf\t%d\n",Likeboot1,Likeboot,(Likeboot1-Likeboot),count1); 
printf(" %lf\t%lf\t%lf\t%d\n",Likeboot1,Likeboot,(Likeboot1-Likeboot),count1);
thistree.start->rate = -1.0*C_underflow*log(thistree.start->rate); 
count1++;
 }  

/* B */
/* boot=b si cambia le basi secondo l' albero
e si scrive nome specie, sequenza, albero e
likelihood */

if (boot=='b') { 
for (k = 0; k < protlen; k++) {
for (i=0; i<Nspecie;i++) {
aaseq1[i][k]=aaseq[i][k];
}}
for (k = 0; k < protlen; k++) {
for (i=Nspecie;i<Nnode;i++) {
    node1 = thistree.pos[i];
for (j=1;j<=3;j++) {
a[0]=node1->index1;
a[1]=l;
a[2]=k;
if ((node1->next->back->leaf)&(node1->next->next->back->leaf)) {
changenode(node1,a); }
node1=node1->next;
}}}
for (i=0;i<Nspecie;i++) {
for (k = 0; k < 10; k++) {
fprintf(verbose, "%c",thistree.pos[i]->seqname[k] ); } 
fprintf(verbose,"\n");
for (k = 0; k < protlen; k++) {
if (aaseq[i][k]=='J') aaseq1[i][k]='-';
fprintf(verbose,"%c", aaseq1[i][k]);
}
fprintf(verbose,"\n");
}
fprintf(verbose,"\n");

pippo=true;
output3(thistree.start->back,pippo);
fprintf(verbose, ";\n");
/*
set_true2(&thistree);
go(&thistree,BIGMLE);

*/
thistree.start->rate= exp(-1.0*X100ACC* (thistree.start->rate));
xpv=calc_like(thistree.start,&thistree);
fprintf(verbose,"%lf\n",xpv );
thistree.start->rate = -1.0*C_underflow*log(thistree.start->rate);
printf("%lf\n",xpv ); 

for (k = 0; k < protlen; k++) {
for (i=0;i<Nspecie;i++) {
aaseq[i][k]=aaseq1[i][k];
}}

} 

}
/* *************************** */
void output2(node1,endprint)
node *node1;
boolean endprint;
{short i, n;
char c;
double rate1;
if (node1->leaf) {   n = 0;
for (i = 0; i < 10; i++) { c = node1->seqname[i];
if (c!=' ') {  
if (endprint) putc(c, outfile);} }
	  } else { 
if (endprint) putc('(', outfile);
	    output2(node1->next->back,endprint);
	    
if (endprint) putc(',', outfile);
	    output2(node1->next->next->back,endprint);
if (node1 == thistree.start->back) {
	    
if (endprint) putc(',', outfile);
	    output2(node1->back,endprint); }
	    
if (endprint) putc(')', outfile);  }
	    rate1 = node1->rate;
if (node1 != thistree.start->back) {
/* if (endprint) fprintf(outfile, ":%lf", rate1*NG); */
if (endprint) fprintf(outfile, ":%lf", rate1);
} 

} 
/* ******************************** */ 
void set_false(tre)
tree *tre;
{short i, j;
  node *node1;
for (i = 0; i < Nspecie; i++) 
thistree.pos[order[i]]->cond2=false;
for (i=Nspecie;i<Nnode;i++) {
    node1 = thistree.pos[i];
for (j=1;j<=3;j++) {
node1->cond2=false;
if (node1->cond1) node1->rate=0.00000000001; 
 node1=node1->next;}}}
/* ****************************** */
void go(tre,BIGMLE)
tree *tre;
double BIGMLE[];
{double threshold;
short i,j;
node *node1;
sw1=0;
threshold=XACC;
BIGMLE[1]=-1.0* LARGE;
BIGMLE[2]=BIGMLE[1]/2.0;
preprocessing(tre);
while(BIGMLE[2] - BIGMLE[1] > threshold*100) {
/* printf("\n %11.5f\t%11.5f\n",BIGMLE[1],BIGMLE[2]); */
printf("\n %11.5f\n",BIGMLE[2]);
BIGMLE[1]=BIGMLE[2];
call_calc_em(BIGMLE,&tre);
}
if (BIGMLE[2]>=BIGMLE[0]) {
BIGMLE[0]=BIGMLE[2];
for (i = 0; i <= Nspecie-1; i++)
	bestorder[i]=order[i];
	printf("copied\n");}}
/* ******************************** */
void result(fine,tre)
tree *tre;
boolean fine;
{ output(thistree.start->back->next->back,fine);
    output(thistree.start->back->next->next->back,fine);
	output(thistree.start,fine);
if (fine) fprintf(outfile, "\n");
	output2(thistree.start->back,fine);
if (fine) fprintf(outfile, ";\n");}
/* ******************************** */
void star(tre,x)
tree *tre;
short x;
{
node *node1;
short k,i;
k=0;
i=Nspecie+x;
link(thistree.pos[i],thistree.pos[order[k]]);
thistree.pos[i] = thistree.pos[i]->next;
k++;
link(thistree.pos[i],thistree.pos[order[k]]);
node1=thistree.pos[i];
while (k<Nspecie-2-x){
node1=node1->next;
i++;
link(thistree.pos[i],node1);
node1 = thistree.pos[i]->next;
k++;
link(node1,thistree.pos[order[k]]);}
node1=node1->next;
link(node1,thistree.pos[order[k+1]]);}
/* **************************** */
void walkinforest()
{ 
boolean endprint;
short i,j,k,i0,i1,j0,j1,k0,k1,jk,ii0,x;
int aa, a[3];

  node *node1;
/* double BIGMLE[3]; */
for (i = 0; i <= Nspecie-1; i++) order[i] = i;
x=0;
BIGMLE[0]=-1.0* LARGE;
BIGMLE[1]=-1.0* LARGE;
BIGMLE[2]=-0.5*LARGE;
partiallike=dmatrix(0,protlen,0,Nstates);
printf("processing data \n");
endprint=false;
if (!usertree) {
set_true(&thistree);
for(x=0;x<Nspecie-3;x++){
BIGMLE[0]=-1.0* LARGE;
for (i0 = 0; i0 < Nspecie-x-1; i0++) {
if (thistree.pos[order[i0]]->aval==0) continue;
for (i1 = i0+1; i1 < Nspecie-x; i1++) {
if (thistree.pos[order[i1]]->aval==0) continue;
i=0;
for (ii0 = 0; ii0 < Nnode; ii0++){
if (thistree.pos[ii0]->aval)
{order[i]=thistree.pos[ii0]->index1-1;
i++;}}
        j1 =1;
        j0=0;
        k0 = order[j0];
        order[j0] = order[i0];
        order[i0] = k0;
        k1 = order[j1];
        order[j1] = order[i1];
        order[i1] = k1;
star(&thistree,x);
set_false(&thistree); 
for (ii0 = 0; ii0 < Nnode; ii0++){
if (thistree.pos[ii0]->aval==1)
{

        thistree.pos[ii0]->back->cond2 = true;
        thistree.pos[ii0]->cond2 = true;
thistree.pos[order[1]]->back->next->cond2 = true;
}}

go(&thistree,BIGMLE);
result(false,&thistree);

for (i = 0; i <= Nspecie-1-x; i++) printf(" %d\t",bestorder[i]);
        printf("\n");
for (i = 0; i <= Nspecie-1-x; i++) printf(" %d\t",order[i]);
for (i = 0; i <= Nspecie-1-x; i++) fprintf(verbose," %d\t",bestorder[i]);
for (i = 0; i <= Nspecie-1-x; i++) fprintf(verbose," %d\t",order[i]);
        set_false(&thistree); }}

for (i = 0; i <= Nspecie; i++) order[i] = bestorder[i];
link(thistree.pos[Nspecie+x],thistree.pos[bestorder[0]]);
thistree.pos[Nspecie+x] = thistree.pos[Nspecie+x]->next;
link(thistree.pos[Nspecie+x],thistree.pos[bestorder[1]]);
thistree.pos[Nspecie+x] = thistree.pos[Nspecie+x]->next;
thistree.pos[bestorder[0]]->aval=false;
thistree.pos[bestorder[1]]->aval=false;
thistree.pos[Nspecie+x]->aval=true;
i=0;
for (ii0 = 0; ii0 < Nnode; ii0++){
if (thistree.pos[ii0]->aval==1)
{order[i]=thistree.pos[ii0]->index1-1;
bestorder[i]=order[i];
printf("\n%d\t%d\t%d\t\t%d%d\n",i,order[i],bestorder[i], order[0],bestorder[0]);
i++;}}}
link(thistree.pos[Nspecie+x],thistree.pos[order[0]]);
thistree.pos[Nspecie+x] = thistree.pos[Nspecie+x]->next;
link(thistree.pos[Nspecie+x],thistree.pos[order[1]]);
thistree.pos[Nspecie+x] = thistree.pos[Nspecie+x]->next;
link(thistree.pos[Nspecie+x],thistree.pos[order[2]]);
set_true(&thistree);
go(&thistree,BIGMLE);
endprint=true;
fprintf(outfile,"\n\n\n"); 
fprintf(outfile," FINAL LIKELIHOOD %8.6lf\n",BIGMLE[0]);
fprintf(outfile,"\n\n\n");
result(true,&thistree);
fprintf(outfile,"\n\n\n"); 
 }
else {
    if (lngths==0) {
     set_true2(&thistree);
     go(&thistree,BIGMLE);}
    else {
    
    thistree.start->rate= exp(-1.0*X100ACC* (thistree.start->rate));
    BIGMLE[0]=calc_like(thistree.start,&thistree);
    thistree.start->rate = -1.0*C_underflow*log(thistree.start->rate);
    printf(" FINAL LIKELIHOOD %8.6lf\n",BIGMLE[0]);
    }
endprint=true;
fprintf(outfile,"\n\n\n");
fprintf(outfile," FINAL LIKELIHOOD %8.6lf\n",BIGMLE[0]);
fprintf(outfile,"\n\n\n");
fprintf(outfile,"\n\n\n");
result(true,&thistree); 
}
printf(" FINAL LIKELIHOOD %8.6lf\n",BIGMLE[0]);
thistree.start->rate= exp(-1.0*X100ACC* (thistree.start->rate));
BIGMLE[1]=calc_like(thistree.start,&thistree);
thistree.start->rate = -1.0*C_underflow*log(thistree.start->rate);
/* printf(" FINAL LIKELIHOOD %8.6lf\t %8.6lf\n",BIGMLE[0],BIGMLE[1]); */

/* printf("lampone2 \n"); */
if (get_cat!='a') prediction(&thistree); 

if (faiboot==true){

for (x = 0; x < NSIMUL; x++){
if (boot=='b') { 

copytree(&thistree,&thattree);     
for (k = 0; k < protlen; k++) {
for (i=0; i<Nspecie;i++) {
aaseq1[i][k]=aaseq[i][k];
}}
for (k = 0; k < protlen; k++) {

for (i=Nspecie;i<Nnode;i++) {
    node1 = thistree.pos[i];
for (j=1;j<=3;j++) {
a[0]=node1->index1;
a[1]=l;
a[2]=k;

ii0=0;
for (j1=0;j1<Nspecie;j1++) {
if (aaseq[j1][k]=='-') ii0=1; 
}
if (ii0==0) {
changenode(node1,a);} 
node1=node1->next;
}}}

for (i=0;i<Nspecie;i++) {
for (k = 0; k < 10; k++) {
fprintf(verbose, "%c",thistree.pos[i]->seqname[k] ); }
fprintf(verbose,"\n");
for (k = 0; k < protlen; k++) {
if (aaseq1[i][k]=='J') aaseq1[i][k]='-';
if (aaseq[i][k]=='J') aaseq1[i][k]='-';
fprintf(verbose,"%c", aaseq1[i][k]);
}
fprintf(verbose,"\n");
}
fprintf(verbose,"\n");

endprint=true;
output3(thistree.start->back,endprint);
fprintf(verbose, ";\n");

set_true2(&thistree);
go(&thistree,BIGMLE);
thistree.start->rate= exp(-1.0*X100ACC* (thistree.start->rate));
BIGMLE[1]=calc_like(thistree.start,&thistree);
thistree.start->rate = -1.0*C_underflow*log(thistree.start->rate);
fprintf(verbose,"%8.6lf\n",BIGMLE[1]);

for (k = 0; k < protlen; k++) {
for (i=0;i<Nspecie;i++) {
aaseq[i][k]=aaseq1[i][k];
}}

copytree(&thattree,&thistree);  
}

if (boot!='b') {
loadingbay();
bootbay();
set_true2(&thistree);
go(&thistree,BIGMLE);

thistree.start->rate= exp(-1.0*X100ACC* (thistree.start->rate));
BIGMLE[1]=calc_like(thistree.start,&thistree);
thistree.start->rate = -1.0*C_underflow*log(thistree.start->rate);
thistree.start=thistree.pos[Nspecie-1];
thistree.start->rate= exp(-1.0*X100ACC* (thistree.start->rate));
Likeboot1=BIGMLE[1];
fprintf(verbose," %lf\t%lf\t%lf\t%d\n",Likeboot1,Likeboot,(Likeboot1-Likeboot),x+1);
printf(" %lf\t%lf\t%lf\t%d\n",Likeboot1,Likeboot,(Likeboot1-Likeboot),x+1);
thistree.start->rate = -1.0*C_underflow*log(thistree.start->rate);
 }


}} 
/* *************************************** */
if (boot=='b') free_cmatrix(aaseq1,0,Nspecie,0,protlen);
free_cmatrix(aaseq,0,Nspecie,0,protlen);
	printf("\noutfile and prolix file saved\n");
	printf("\n\ndone\n\n");
free_dmatrix(partiallike,0,protlen,0,Nstates);
	free_dmatrix(Lmat,0,Nstru,0,naa);
	free_d3matrix(Imat,Nstru,naa,naa);
	free_d3matrix(Umat,Nstru,naa,naa);
	free_dmatrix(Phmm,0,Nstates,0,Nstates);
	free_dmatrix(aafreq,0,Nstru,0,naa);
	free_dvector(structfreq,0,Nstates);
	free_ivector(thiscat,0,Nstates);
	free_ivector(State2ss,0,Nstates);

if (structure) free_cvector(dsspseq,0,protlen-1);
if (accessibility) free_cvector(accseq,0,protlen-1);


for (i=0;i<Nspecie;i++) {
for (j=0;j<protlen;j++)
    free(thistree.pos[i]->sitevalue[j]);
    free(thistree.pos[i]->sitevalue);  }
for (i=Nspecie;i<Nnode;i++) {node1 = thistree.pos[i];
for (j=1;j<=3;j++) {
for (k=0;k<protlen; k++)
    free(node1->sitevalue[k]);
    free(node1->sitevalue);
    node1 = node1->next; }}
if (boot=='b') {
for (i=0;i<Nspecie;i++) {
for (j=0;j<protlen;j++)
    free(thattree.pos[i]->sitevalue[j]);
    free(thattree.pos[i]->sitevalue);  }
for (i=Nspecie;i<Nnode;i++) {node1 = thattree.pos[i];
for (j=1;j<=3;j++) {
for (k=0;k<protlen; k++)
    free(node1->sitevalue[k]);
    free(node1->sitevalue);
    node1 = node1->next; }}
}


	fclose(infile);
	fclose(outfile);
	fclose(verbose);
return; } 

